from flask import session
from flask import request, Blueprint, jsonify
from sqlalchemy import create_engine, select, MetaData, Table,text
from sqlalchemy.sql import and_, or_

from core import app

engine = create_engine(app.config['DATABASE_URI'],pool_pre_ping=True,pool_recycle=3600)

class RegdeskModel():  
	def __init__(self):
		try:
			self.meta = MetaData()
			self.states    = Table("states", self.meta, autoload=True, autoload_with=engine)
			self.conference  = Table("conference",self.meta,autoload = True, autoload_with= engine)	
			self.delegates  = Table("delegates",self.meta,autoload = True, autoload_with= engine)
			
		except Exception as e:
			print(e)


	def generateDelegatePresent(self):
		conn = engine.connect()
		stmt = text("select * from delegates_iages23 t inner join delegates d on d.delegate_no = t.delegate_no where d.conference_id = 1 and t.signed_on is not null and d.is_present is null; ")
		results = conn.execute(stmt)
		conn.close()
		if results : 
			return results
		else:
			return None 		

	def getConfDataForRegdesk(self,regdesk_pin):
		conn = engine.connect()
		# stmt = text("select conf_id,conf_name, from conference where  conf_key = '"+conf_key+"'  limit 1;")
		stmt = text("SELECT conf_id,conf_name,conf_title,conf_key,username,regdesk_pin,conf_start_time,conf_end_time,del_table_name,is_regdesk_open FROM conference where regdesk_pin = "+str(regdesk_pin)+" and is_regdesk_open > 0  order by conf_id desc limit 1;")
		results = conn.execute(stmt)
		conn.close()
		results = [dict(r) for r in results] if results else None
		if results : 
			return results[0]
		else:
			return None 	

	def delegateData(self,conf_id,del_table_name):
		conn = engine.connect()
		stmt = text("select delegate_id,delegate_no,role,full_name,email,mobile,city,state,membership_no,mc_number, counter,batch,signed_by,comments,signed_on,updated_at,conf_id from "+del_table_name+"  where conf_id = "+str(conf_id)+
			" and delegate_no > 0 and delegate_no is not null and full_name is not null and role not in ('VOLUNTEER','TRADE','EXHIBIOR','CREW','SUPPORT - IT','SUPPORT - AV') limit 10 ;")
		# stmt = text("select * from  "+del_table_name+"  where conf_id = "+str(conf_id)+" and delegate_no > 0 and delegate_no is not null  ;")
		results = conn.execute(stmt)
		conn.close()
		results = [dict(r) for r in results] if results else None
		if results : 
			return results
		else:
			return None 				

	# def updatRegdeskSignedData(self,del_table_name,update_query,delegate_no_str,device_id):
	# 	conn    = engine.connect()
	# 	try:
	# 		stmt        = text(update_query)
	# 		safe_update = text("SET SQL_SAFE_UPDATES = 0 ;")
	# 		result      = conn.execute(safe_update)
	# 		results     = conn.execute(stmt)
	# 		stmt_2      = text("select * from " +del_table_name+ " where delegate_no not in ("+delegate_no_str+") and device_id != '"+device_id+"'  and sync_at is not null ; ")

	# 		results_3   = conn.execute(stmt_2)
	# 		signed_data = [dict(r) for r in results_3] if results_3 else None
	# 		conn.close()
	# 		if signed_data : 
	# 			return signed_data
	# 		else:
	# 			return None 
	# 	except Exception as e:
	# 		conn.close()
	# 		return str(e)

	# get other device signed in records
	def getRegdeskSignedData(self,del_table_name,delegate_no_str):
		conn    = engine.connect()
		try:
			if delegate_no_str :
				stmt = text("select * from " +del_table_name+ " where delegate_no not in ("+delegate_no_str+") and device_id !='"+device_id+"' and sync_at is not null ; ")
			else : 
				stmt    = text("select * from " +del_table_name+ " where sync_at is not null ; ")

			print(stmt)
			results = conn.execute(stmt)
			signed_data = [dict(r) for r in results] if results else None
			conn.close()
			if signed_data : 
				return signed_data
			else:
				return None 
		except Exception as e:
			return str(e)	

	# Feb 25 2023 ----------------
	# Multiple result set
	def updateAndGetSignedCount(self,conf_id,delegate_no,del_table_name,curr_dt,signed_by,comments):
		sets = []
		try:
			connection = engine.raw_connection()
			cursor     = connection.cursor()
			# cursor = connection.cursor()
			cursor.callproc("update_android_sync_data",[conf_id,delegate_no,del_table_name,curr_dt,signed_by,comments])
			while 1:
				names = [c[0] for c in cursor.description]
				set_  = []
				while 1:
					row_raw = cursor.fetchone()
					if row_raw is None:
						break
						
					row = dict(zip(names, row_raw))
					set_.append(row)
	 
				sets.append(list(set_))
	 
				if cursor.nextset() is None:
					break
	 
				# nextset() doesn't seem to be sufficiant to tell the end.
				if cursor.description is None:
					break

			cursor.close()
			connection.commit()		
		finally:
			# Return the connection to the pool (won't actually close).
			connection.close()
			
		return sets
					
	# Multiple result set
	def syncAllDataWithSignedCount(self,conf_id,del_table_name,date_1):
		sets = []
		try:
			connection = engine.raw_connection()
			cursor     = connection.cursor()
			cursor.callproc("usp_android_sync_with_signed_count",[conf_id,del_table_name,date_1])
			while 1:
				names = [c[0] for c in cursor.description]
				set_  = []
				while 1:
					row_raw = cursor.fetchone()
					if row_raw is None:
						break
						
					row = dict(zip(names, row_raw))
					set_.append(row)
	 
				sets.append(list(set_))
	 
				if cursor.nextset() is None:
					break
	 
				# nextset() doesn't seem to be sufficiant to tell the end.
				if cursor.description is None:
					break

			cursor.close()
			connection.commit()		
		finally:
			# Return the connection to the pool (won't actually close).
			connection.close()
			
		return sets
					


